---
header-includes: |
  \usepackage[T1]{fontenc}
  \usepackage[a4paper,bindingoffset=0in,left=4cm,right=2cm,top=2cm,bottom=2cm,footskip=.25in]{geometry}
  \usepackage{fancyhdr}
  \usepackage[utf8]{inputenc}
  \usepackage{tikz}
  \usetikzlibrary{snakes,arrows,shapes,shadows,calc}
  \usepackage{amsmath}
  \usepackage{pgfplots}
  \usepackage{pgfplotstable}
  \pgfplotsset{compat=1.8}
  \usepackage{multicol,lipsum}
  \usepackage{listings}
  \usepackage{wrapfig}
  \usepackage[font=small,labelfont=bf,tableposition=top, position=above]{caption}
  \usepackage{graphicx}
  \usepackage{xcolor,stackengine}
  \usepackage{colortbl}
  \usepackage[most]{tcolorbox}
  \usepackage{lastpage}
  \usepackage{subcaption}
  \usepackage{tabularx}
  \usepackage{biblatex}
  \usepackage{float}
  \usepackage{refract}
  \usepackage{svg}
  \usepackage{setspace}
  \usepackage{titling}
  \usepackage{abstract}
	\usepackage{fvextra}
	\usepackage{emoji}
  \usepackage{fontspec}
  \usepackage{dirtree}
  \usepackage{etoolbox}
  \usepackage{textpos}
  \setmainfont{Times New Roman}
	\DefineVerbatimEnvironment{Highlighting}{Verbatim}{breaklines,commandchars=\\\{\}}
  \newcommand{\hideFromPandoc}[1]{#1}
  \hideFromPandoc{
    \let\Begin\begin
    \let\End\end
  }
  \makeatletter
  \def\fps@figure{h}
  \makeatother
---

\captionsetup[figure]{position=above}

\setstretch{1.5}

\setemojifont{Twemoji Mozilla}

<!-- \renewcommand{\familydefault}{\sfdefault} -->

\graphicspath{ {./img/} }

\overfullrule=0pt

\newcolumntype{C}{>{\centering\arraybackslash}X}

\renewcommand{\comment}[1]{\hspace{2em}{\small\textit{#1}}\bigskip\par}

\renewcommand{\lstlistingname}{Auflistung}

\newcommand\palbox[1]{{\sffamily\fboxsep=5pt\relax\fboxrule=1pt\relax\footnotesize%
  \fcolorbox{gray!50}{gray!10}{%
    \stackengine{4pt}{%
      \colorbox[HTML]{#1}{\rule{30pt}{0pt}\rule{0pt}{30pt}}%
}{%
\color{black!60}\stackengine{6pt}{\##1}{\saycolors{#1}}{U}{l}{F}{F}{S}%
}{U}{l}{F}{F}{S}%
}%
}}
\newcommand\saycolors[1]{\relax}

\lstset{
basicstyle=\ttfamily,
columns=fullflexible,
frame=single,
breaklines=true,
postbreak=\mbox{\textcolor{red}{$\hookrightarrow$}\space},
}

\apptocmd{\dirtree}{\bigskip}{}{}
\pretocmd{\dirtree}{\bigskip}{}{}

\newcommand{\mkbibnodate}{n\adddot d\adddot}

\setlength{\parindent}{0em}

\def\shadowshift{3pt,-3pt}
\def\shadowradius{6pt}

\colorlet{innercolor}{black!60}
\colorlet{outercolor}{gray!05}

\newcommand\drawshadow[1]{
\begin{pgfonlayer}{shadow}
\shade[outercolor,inner color=innercolor,outer color=outercolor] ($(#1.south west)+(\shadowshift)+(\shadowradius/2,\shadowradius/2)$) circle (\shadowradius);
\shade[outercolor,inner color=innercolor,outer color=outercolor] ($(#1.north west)+(\shadowshift)+(\shadowradius/2,-\shadowradius/2)$) circle (\shadowradius);
\shade[outercolor,inner color=innercolor,outer color=outercolor] ($(#1.south east)+(\shadowshift)+(-\shadowradius/2,\shadowradius/2)$) circle (\shadowradius);
\shade[outercolor,inner color=innercolor,outer color=outercolor] ($(#1.north east)+(\shadowshift)+(-\shadowradius/2,-\shadowradius/2)$) circle (\shadowradius);
\shade[top color=innercolor,bottom color=outercolor] ($(#1.south west)+(\shadowshift)+(\shadowradius/2,-\shadowradius/2)$) rectangle ($(#1.south east)+(\shadowshift)+(-\shadowradius/2,\shadowradius/2)$);
\shade[left color=innercolor,right color=outercolor] ($(#1.south east)+(\shadowshift)+(-\shadowradius/2,\shadowradius/2)$) rectangle ($(#1.north east)+(\shadowshift)+(\shadowradius/2,-\shadowradius/2)$);
\shade[bottom color=innercolor,top color=outercolor] ($(#1.north west)+(\shadowshift)+(\shadowradius/2,-\shadowradius/2)$) rectangle ($(#1.north east)+(\shadowshift)+(-\shadowradius/2,\shadowradius/2)$);
\shade[outercolor,right color=innercolor,left color=outercolor] ($(#1.south west)+(\shadowshift)+(-\shadowradius/2,\shadowradius/2)$) rectangle ($(#1.north west)+(\shadowshift)+(\shadowradius/2,-\shadowradius/2)$);
\filldraw ($(#1.south west)+(\shadowshift)+(\shadowradius/2,\shadowradius/2)$) rectangle ($(#1.north east)+(\shadowshift)-(\shadowradius/2,\shadowradius/2)$);
\end{pgfonlayer}
}

\pgfdeclarelayer{shadow}
\pgfsetlayers{shadow,main}

\newsavebox\mybox
\newlength\mylen

\newcommand\shadowimage[2][]{%
\setbox0=\hbox{\includegraphics[#1]{#2}}
\setlength\mylen{\wd0}
\ifnum\mylen<\ht0
\setlength\mylen{\ht0}
\fi
\divide \mylen by 120
\def\shadowshift{\mylen,-\mylen}
\def\shadowradius{\the\dimexpr\mylen+\mylen+\mylen\relax}
\begin{tikzpicture}
\node[anchor=south west,inner sep=0] (image) at (0,0) {\includegraphics[#1]{#2}};
\drawshadow{image}
\end{tikzpicture}}
